local mvec3_copy = mvector3.copy
local dis_sq = mvector3.distance_sq

Hooks:PostHook(PlayerManager,"check_skills","kingslayer_playermanager_checkskills",function(self)
	if self:has_category_upgrade("player","kingslayer_syringe_rof_bonus") then 
		self:set_property("kingslayer_rof_bonus_stacks",0)
	end
	if self:has_category_upgrade("player","kingslayer_syringe_spread_bonus") then 
		self:set_property("kingslayer_spread_bonus",0)
	end
	
	if self:has_category_upgrade("player","kingslayer_syringe_base") then 
		local base_upgrade_data = self:upgrade_value("player","kingslayer_syringe_base")
		local cooldown_drain = base_upgrade_data.cooldown_drain_per_kill
		
		
		self:register_message(Message.OnEnemyKilled,"kingslayer_syringe_cooldown_drain_on_kill",
			function(equipped_unit,variant,killed_unit)
				local player = self:local_player()
				if alive(player) then
					managers.player:speed_up_grenade_cooldown(cooldown_drain)
				end
			end
		)
		
	else
		self:unregister_message(Message.OnEnemyKilled,"kingslayer_syringe_cooldown_drain_on_kill")
	end
	
	if self:has_category_upgrade("player","kingslayer_health_on_kill") then 
		local upgrade_data = self:upgrade_value("player","kingslayer_health_on_kill")
		local gain_health_percent = upgrade_data.health_restored
		local cooldown = upgrade_data.cooldown
		
		self:register_message(Message.OnEnemyKilled,"kingslayer_on_enemy_killed_restore_health",
			function(equipped_unit,variant,killed_unit)
				if not self:has_active_temporary_property("kingslayer_health_on_kill_cooldown") then 
					local player = self:local_player()
					if alive(player) then
						self:activate_temporary_property("kingslayer_health_on_kill_cooldown",cooldown,true) --set effect cooldown timer
						local dmg_ext = player:character_damage()
						dmg_ext:restore_health(gain_health_percent,false,true)
					end
				end
			end
		)
	else
		self:unregister_message(Message.OnEnemyKilled,"kingslayer_on_enemy_killed_restore_health")
	end
	
	if self:has_category_upgrade("player","kingslayer_stacks_on_kill") then 
		local upgrade_data = self:upgrade_value("player","kingslayer_stacks_on_kill")
		local max_stacks = upgrade_data.max_stacks
		local stacks_per_kill = upgrade_data.stacks_per_kill
		local duration = upgrade_data.duration
		
		self:register_message(Message.OnEnemyKilled,"kingslayer_on_enemy_killed_gain_stacks",
			function(equipped_unit,variant,killed_unit)
				local player = self:local_player()
				if alive(player) then
					local num_kingslayer_stacks = math.min(self:get_temporary_property("kingslayer_stacks",0) + stacks_per_kill,max_stacks)
					
					self:activate_temporary_property("kingslayer_stacks",duration,num_kingslayer_stacks)
				end
			end
		)
	else
		self:unregister_message(Message.OnEnemyKilled,"kingslayer_on_enemy_killed_gain_stacks")
	end
end)

local orig_damage_resist = PlayerManager.damage_reduction_skill_multiplier
function PlayerManager:damage_reduction_skill_multiplier(...)
	local bonus_resist = 0 
	if self:has_category_upgrade("player","kingslayer_stacks_to_damage_resist") then 
		bonus_resist = self:get_temporary_property("kingslayer_stacks",0) * self:upgrade_value("player","kingslayer_stacks_to_damage_resist",0)
	end
	return orig_damage_resist(self,...) - bonus_resist
end


function PlayerManager:_attempt_kingslayer_syringe()
	if self:has_category_upgrade("player","kingslayer_syringe_base") then 

--		local cooldown_per_kill = upgrade_data.cooldown_per_kill
		
--		local now = managers.game_play_central:get_heist_timer()
--		managers.network:session():send_to_peers("sync_ability_hud", now + duration, duration)
		
		
		if self._coroutine_mgr:is_running("kingslayer_syringe") then
			return false
		end

		self:add_coroutine("kingslayer_syringe", PlayerAction.KingslayerSyringe)
		return true
	end
	return false
end






--networking fixes (to not crash other players)

local function get_as_digested(amount)
	local list = {}

	for i = 1, #amount do
		table.insert(list, Application:digest_value(amount[i], false))
	end

	return list
end

local function make_double_hud_string(a, b)
	return string.format("%01d|%01d", a, b)
end

local function add_hud_item(amount, icon)
	if #amount > 1 then
		managers.hud:add_item_from_string({
			amount_str = make_double_hud_string(amount[1], amount[2]),
			amount = amount,
			icon = icon
		})
	else
		managers.hud:add_item({
			amount = amount[1],
			icon = icon
		})
	end
end

local function set_hud_item_amount(index, amount)
	if #amount > 1 then
		managers.hud:set_item_amount_from_string(index, make_double_hud_string(amount[1], amount[2]), amount)
	else
		managers.hud:set_item_amount(index, amount[1])
	end
end

function PlayerManager:update_grenades_to_peer(peer)
	local peer_id = managers.network:session():local_peer():id()

	if self._global.synced_grenades[peer_id] then
		local grenade = self._global.synced_grenades[peer_id].grenade
		local tweak = tweak_data.blackmarket.projectiles[grenade]
		if tweak.based_on then 
			grenade = tweak.based_on
		end
		local amount = self._global.synced_grenades[peer_id].amount

		peer:send_queued_sync("sync_grenades", grenade, Application:digest_value(amount, false), 0)
	end
end

function PlayerManager:update_grenades_amount_to_peers(grenade, amount, register_peer_id)
	local peer_id = managers.network:session():local_peer():id()
	
		local tweak = tweak_data.blackmarket.projectiles[grenade]
		if tweak.based_on then 
			grenade = tweak.based_on
		end
		
	managers.network:session():send_to_peers_synched("sync_grenades", grenade, amount, register_peer_id or 0)
	self:set_synced_grenades(peer_id, grenade, amount, register_peer_id)
end

--fixes the synced (spoofed) grenade data being used for the actual data- specifically, cooldown
function PlayerManager:add_grenade_amount(amount, sync)
	local peer_id = managers.network:session():local_peer():id()
	local grenade = managers.blackmarket:equipped_grenade() --self._global.synced_grenades[peer_id].grenade
	local tweak = tweak_data.blackmarket.projectiles[grenade]
	local max_amount = self:get_max_grenades_by_peer_id(peer_id)
	local icon = tweak.icon
	local previous_amount = self._global.synced_grenades[peer_id].amount

	if amount > 0 and tweak.base_cooldown then
		managers.hud:animate_grenade_flash(HUDManager.PLAYER_PANEL)
	end

	amount = math.min(Application:digest_value(previous_amount, false) + amount, max_amount)

	if amount < max_amount and tweak.base_cooldown then
		self:replenish_grenades(tweak.base_cooldown)
	end

	managers.hud:set_teammate_grenades_amount(HUDManager.PLAYER_PANEL, {
		icon = icon,
		amount = amount
	})
	self:update_grenades_amount_to_peers(grenade, amount, sync and peer_id)
end
